describe( "Inviting a new team member", () => {

  before(() => {
    API.Workflow.registerOrganization(organization);
    // disable tour sets a cookie
    disableTour(Tours.ROOMS_LIST);
    disableTour(Tours.TEAM);
  });

  it("should be able to invite a staff member", () => {
    const newTeamMember = Person.StaffMember;
    cy.log(organization.subDomain);

    UI.loginAs(organization.owner, organization)
      .then(TeamPage.open)
      .then(TeamPage.startNewInvite)
      .then(TeamPage.Invite.cannotBeSent)   // should not be able to invite a new team member without a name
      .then(TeamPage.Invite.toPerson(newTeamMember.fullName))
      .then(TeamPage.Invite.cannotBeSent)   // should not be able to invite a new team member without an email address
      .then(TeamPage.Invite.withEmail(newTeamMember.emailAddress))
      .then(TeamPage.Invite.send)
      .then(TeamPage.Member.find(newTeamMember))
      .then(TeamPage.Member.hasRole(newTeamMember.role))
      .then(TeamPage.Member.hasStatus("Pending"));
  });
});


export function registerOrganization(organization: Organization): void {
  login(Person.SuperAdministrator, Organization.Admin);
  inviteOrganizationToPlatform(organization).then( () => {
    cy.logout(organization.subDomain);
    cy.switchOrganization(organization.subDomain);

    completeUserRegistration(organization.owner, organization.inviteId);
    login(organization.owner, organization).then( () => {
      completeAccountRegistration(organization);
    });

  });
}


export function loginAs(person: Person, organization: Organization): Cypress.Chainable {
  return cy.switchOrganization(organization.subDomain)
    .then(() => cy.logout(organization.subDomain))
    .then(() => cy.visit(loginPage(organization)))
    .then(login(person))
    .then(setAuthorizationCookie(person, organization))
    .end();
}

// This is used if any calls are made to the API
const setAuthorizationCookie = (person: Person, organization: Organization) => (): Cypress.Chainable => {
  return cy.getCookie(`__jwt${organization.subDomain}`)
    .then((cookie: any) => {
      if (cookie) {
        person.credential.authorizationCookie = `${cookie.name}=${cookie.value}`;
      }
    });
};

// We did at one point edit the cookies in the window.document directly but have
// since removed it to see if we could get a more traditional approach of logging in via
// UI components working
const login = (person: Person) => (): Cypress.Chainable => {
  return cy.get("#emailInput").type(person.emailAddress)
    .get("#passwordInput").type(person.credential.password)
    .get("#mainTehamaLogin").click()
    .get("#codeInput").type(person.credential.multiFactorCode)
    .get("#mainVelaLogin").click();
};

// We switch organization during the test between "app" and "org" (variable)
// This function changes the baseUrl so we don't need to be explicit when using
// `visit` and `request`
function switchOrganization(newOrganization: string): Cypress.Chainable {
  const cypressBaseUrl = Cypress.config().baseUrl;
  if (cypressBaseUrl && cypressBaseUrl.indexOf(newOrganization) === -1) {
    Cypress.config().baseUrl = cypressBaseUrl.replace(currentOrganization, newOrganization);
    currentOrganization = newOrganization;
    Cypress.log( {
      name: "switch_org",
      message: [`Switched to organization '${currentOrganization}'`]
    }).end();
  }
  return cy;
}